/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.rjconsultores.ventaboletos.web.gui.controladores.seguridad;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Textbox;

import com.rjconsultores.ventaboletos.entidad.Perfil;
import com.rjconsultores.ventaboletos.entidad.Usuario;
import com.rjconsultores.ventaboletos.service.PerfilService;
import com.rjconsultores.ventaboletos.service.UsuarioService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.vo.segurida.PerfilJerarquia;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderPerfil;
import com.trg.search.Filter;

/**
 * 
 * @author rodrigo
 */
@Controller("busquedaPerfilController")
@Scope("prototype")
public class BusquedaPerfilController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	private static Logger log = LogManager.getLogger(BusquedaPerfilController.class);

	@Autowired
	private transient PagedListWrapper<Perfil> plwPerfil;
	@Autowired
	private PerfilService perfilService;
	@Autowired
	private UsuarioService usuarioService;

	private MyListbox perfilList;
	private Paging pagingPerfil;
	private Textbox txtNome;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);

		perfilList.setItemRenderer(new RenderPerfil());

		perfilList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				Perfil s = (Perfil) perfilList.getSelected();
				verSistema(s);
			}
		});

		refreshLista();

		txtNome.focus();
	}

	private void verSistema(Perfil s) throws InterruptedException {
		if (s == null) {
			return;
		}

		Map args = new HashMap();
		args.put("perfil", s);
		args.put("perfilList", perfilList);

		openWindow("/gui/seguridad/editarPerfil.zul",
				Labels.getLabel("editarPerfilController.window.title"), args, MODAL);
	}

	private void refreshLista() {
		HibernateSearchObject<Perfil> sistemaBusqueda =
				new HibernateSearchObject<Perfil>(Perfil.class,
						pagingPerfil.getPageSize());

		String descPerfil = txtNome.getText();
		if (descPerfil != null) {
			sistemaBusqueda.addFilterLike("descperfil", "%" + descPerfil.trim().concat("%"));
		}

		sistemaBusqueda.addSortAsc("descperfil");
		sistemaBusqueda.addFilterEqual(ACTIVO, Boolean.TRUE);

		Usuario usuarioLogado = usuarioService.obtenerID(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		Perfil perfilUsuario = usuarioLogado.getUsuarioPerfilList().get(0).getPerfil();
		PerfilJerarquia usuarioJerarquia = perfilUsuario.getIndJerarquia() == null ? PerfilJerarquia.NORMAL : PerfilJerarquia.buscar(perfilUsuario.getIndJerarquia());

		if (!usuarioJerarquia.equals(PerfilJerarquia.ADMIN)) {
			if (!usuarioJerarquia.equals(PerfilJerarquia.NORMAL)) {
				sistemaBusqueda.addFilterOr(
						new Filter("indJerarquia", usuarioJerarquia.getValor(), Filter.OP_GREATER_OR_EQUAL),
						new Filter("indJerarquia", null, Filter.OP_NULL),
						new Filter("indJerarquia", PerfilJerarquia.NORMAL.getValor(), Filter.OP_LESS_OR_EQUAL));
			} else {
				sistemaBusqueda.addFilterOr(
						new Filter("indJerarquia", null, Filter.OP_NULL),
						new Filter("indJerarquia", PerfilJerarquia.NORMAL.getValor(), Filter.OP_LESS_OR_EQUAL));
			}
		}

		plwPerfil.init(sistemaBusqueda, perfilList, pagingPerfil);
		List<Perfil> perfilRJ = perfilService.buscar("RJ");
		
		if ( (perfilRJ != null) && !perfilRJ.isEmpty()){
			perfilList.removeItem(perfilService.buscar("RJ").get(0));
		}
		if (perfilList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaPerfilController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
				log.error("", ex);
			}
		}

	}

	public void onClick$btnPesquisa(Event ev) {
		refreshLista();
	}

	public void onClick$btnRefresh(Event ev) {
		refreshLista();
	}

	public void onClick$btnNovo(Event ev) throws InterruptedException {
		verSistema(new Perfil());
	}
}
